/**
 * Validator for classification exceptions
 * @param y_true
 * @param y_pred
 * @param labels
 * @param options
 * @ignore
 */
export declare const validateInitialInputs: (y_true: any, y_pred: any, labels: any, options?: {}) => void;
/**
 * Accuracy classification score.
 *
 * In multilabel classification, this function computes subset accuracy:
 * the set of labels predicted for a sample must exactly match the corresponding set of labels in y_true.
 *
 * @example
 * import { accuracyScore } from 'kalimdor/metrics';
 *
 * const accResult = accuracyScore({
 *   y_true: [0, 1, 2, 3],
 *   y_pred: [0, 2, 1, 3]
 * });
 *
 * // accuracy result: 0.5
 *
 * @param {any} y_true
 * @param {any} y_pred
 * @param {any} normalize
 * @param {any} sample_weight
 */
export declare function accuracyScore({ y_true, y_pred, normalize }: {
    y_true: any;
    y_pred: any;
    normalize?: boolean;
}): number;
/**
 * Zero-one classification loss.
 *
 * If normalize is `true`, return the fraction of misclassifications (float),
 * else it returns the number of misclassifications (int). The best performance is 0.
 *
 * @example
 * import { zeroOneLoss } from 'kalimdor/metrics';
 *
 * const loss_zero_one_result = zeroOneLoss({
 *   y_true: [1, 2, 3, 4],
 *   y_pred: [2, 2, 3, 5]
 * });
 * console.log(loss_zero_one_result); // 0.5
 *
 * @param {any} y_true
 * @param {any} y_pred
 * @param {any} normalize
 * @returns {number}
 */
export declare function zeroOneLoss({ y_true, y_pred, normalize }: {
    y_true: any;
    y_pred: any;
    normalize?: boolean;
}): number;
export interface ConfusionMatrixOptions {
    /**
     * Ground truth (correct) target values.
     */
    y_true: any[];
    /**
     * Estimated targets as returned by a classifier.
     */
    y_pred: any[];
    /**
     * List of labels to index the matrix. This may be used to reorder or
     * select a subset of labels. If none is given, those that appear
     * at least once in y_true or y_pred are used in sorted order.
     */
    labels?: any[];
}
/**
 * A confusion matrix is a technique for summarizing the performance of a classification algorithm.
 *
 * Classification accuracy alone can be misleading if you have an unequal number of observations in each class or if you have more than two classes in your dataset.
 *
 * Calculating a confusion matrix can give you a better idea of what your classification model is getting right and what types of errors it is making.
 *
 * @example
 * import { confusion_matrix } from 'kalimdor/metrics';
 *
 * const matrix1 = confusion_matrix({
 *   y_true: [1, 2, 3],
 *   y_pred: [1, 2, 3]
 * });
 * console.log(matrix1); // [ [ 1, 0, 0 ], [ 0, 1, 0 ], [ 0, 0, 1 ] ]
 *
 * const matrix2 = confusion_matrix({
 *   y_true: ['cat', 'ant', 'cat', 'cat', 'ant', 'bird'],
 *   y_pred: ['ant', 'ant', 'cat', 'cat', 'ant', 'cat']
 * });
 * console.log(matrix2); // [ [ 1, 2, 0 ], [ 2, 0, 0 ], [ 0, 1, 0 ] ]
 *
 * @param {ConfusionMatrixOptions} options
 * @returns {number[]}
 */
export declare function confusion_matrix(options: ConfusionMatrixOptions): number[];
