export interface KMeansOptions {
    /**
     * Number of clusters
     */
    k: number;
    /**
     * Choice of distance method. Defaulting to euclidean
     */
    distance?: 'euclidean' | 'manhattan';
    /**
     * Relative tolerance with regards to inertia to declare convergence
     */
    maxIteration?: number;
    /**
     * Random state value for sorting centroids during the getInitialCentroid phase
     */
    randomState?: number;
}
/**
 * K-Means clustering
 *
 * @example
 * import { KMeans } from 'kalimdor/cluster';
 *
 * const kmean = new KMeans({ k: 2 });
 * const clusters = kmean.fit({ X: [[1, 2], [1, 4], [1, 0], [4, 2], [4, 4], [4, 0]] });
 *
 * const result = kmean.predict({ X: [[0, 0], [4, 4]] });
 * // results in: [0, 1]
 */
export declare class KMeans {
    private assignment;
    private centroids;
    private clusters;
    private distance;
    private k;
    private randomState;
    private maxIteration;
    constructor(options?: KMeansOptions);
    /**
     * Compute k-means clustering.
     * @param {any} X
     * @returns {{centroids: number[]; clusters: number[]}}
     */
    fit({ X }: {
        X: any;
    }): void;
    /**
     * Predicts the cluster index with the given X
     * @param {any} X
     * @returns {number[]}
     */
    predict({ X }: {
        X: any;
    }): number[];
    /**
     * Get the model details in JSON format
     * @returns {{k: number; clusters: number[]; centroids: number[]}}
     */
    toJSON(): {
        k: number;
        clusters: number[];
        centroids: number[];
    };
    /**
     * Restores the model from checkpoints
     * @param {number} k
     * @param {number[]} clusters
     * @param {number[]} centroids
     */
    fromJSON({ k, clusters, centroids }: {
        k: number;
        clusters: number[];
        centroids: number[];
    }): void;
    /**
     * Get initial centroids from X of k
     * @param {number[]} X
     * @param {number} k
     * @returns {number[]}
     */
    private getInitialCentroids;
    /**
     * Get closest centroids based on the passed in distance method
     * @param {number[]} data
     * @param {number[]} centroids
     * @param distance
     * @returns {number}
     */
    private getClosestCentroids;
}
